import {
  Button,
  Col,
  Collapse,
  CollapseProps,
  Form,
  Input,
  Modal,
  Row,
  Space,
  Tooltip,
  theme,
} from "antd";
import {
  CategoryItemType,
  LabelTreeProps,
  LabelType,
  ModalType,
} from "./interface";
import { useEffect, useState } from "react";
import { defaultModalConfig, modalConfig } from "./config";
import {
  CaretRightOutlined,
  CloseOutlined,
  EditOutlined,
  FormOutlined,
  TagOutlined,
} from "@ant-design/icons";

export const LabelTree = (props: LabelTreeProps) => {
  const { token } = theme.useToken();
  const [modalType, setModalType] = useState<ModalType | null>(null);
  const [form] = Form.useForm<{ name: string }>();
  const [activeLabel, setActiveLabel] = useState<LabelType | null>(null);
  const [activeCategory, setActiveCategory] = useState<CategoryItemType | null>(
    null
  );

  useEffect(() => {
    switch (modalType) {
      case "updateLabel":
        form.setFieldsValue({ name: activeLabel?.name });
        break;
      case "updateCategory":
        form.setFieldsValue({ name: activeCategory?.name });
        break;
      default:
        break;
    }
  }, [activeLabel, activeCategory, form, modalType]);

  const { title, placeholder } = !modalType
    ? defaultModalConfig
    : modalConfig[modalType];

  const handleConfirm = async () => {
    try {
      await form.validateFields();
      const { name } = form.getFieldsValue();
      switch (modalType) {
        case "createLabel":
          activeCategory &&
            props.onCreateLabel?.({ name, categoryId: activeCategory.id });
          break;
        case "updateLabel":
          activeLabel && props.onUpdateLabel?.({ name, id: activeLabel.id });
          break;
        case "updateCategory":
          activeCategory &&
            props.onUpdateCategory?.({ name, id: activeCategory.id });
          break;
        default:
          break;
      }
      form.resetFields();
      setModalType(null);
    } catch (error) {
      console.log(error);
    }
  };

  const getItems = (dataList: CategoryItemType[]): CollapseProps["items"] =>
    dataList
      .map((item) => {
        return {
          key: item.id,
          label: (
            <Row>
              <Col span={18}>{item.name}</Col>
              <Col span={6}>
                <Space>
                  <Tooltip title={`编辑 ${item.name} 分类`}>
                    <FormOutlined
                      onClick={(e) => {
                        e.stopPropagation();
                        setActiveCategory(item);
                        setModalType("updateCategory");
                      }}
                    />
                  </Tooltip>
                  <Tooltip title={`删除 ${item.name} 分类`}>
                    <CloseOutlined />
                  </Tooltip>
                </Space>
              </Col>
            </Row>
          ),
          children: (
            <ul className="label-catalog">
              {item.labels?.map((label) => (
                <li key={label.id}>
                  <Row>
                    <Col span={18}>{label.name}</Col>
                    <Col span={6}>
                      <Space>
                        <Tooltip title={`编辑 ${label.name} 标签`}>
                          <EditOutlined
                            onClick={(e) => {
                              e.stopPropagation();
                              setActiveLabel(label);
                              setModalType("updateLabel");
                            }}
                          />
                        </Tooltip>
                        <Tooltip title={`删除 ${label.name} 标签`}>
                          <CloseOutlined
                            onClick={(e) => {
                              e.stopPropagation();
                              props.onDeleteLabel?.(label);
                            }}
                          />
                        </Tooltip>
                      </Space>
                    </Col>
                  </Row>
                </li>
              ))}
              <Button
                type="dashed"
                block
                icon={<TagOutlined />}
                onClick={() => {
                  setActiveCategory(item);
                  setModalType("createLabel");
                }}
              >
                添加标签
              </Button>
            </ul>
          ),
        };
      })
      .map((i) => ({
        ...i,
        headerClass: "label-category-header",
        className: "label-category-item",
      }));

  return (
    <div>
      <Collapse
        bordered={false}
        defaultActiveKey={[]}
        expandIcon={({ isActive }) => (
          <CaretRightOutlined rotate={isActive ? 90 : 0} />
        )}
        style={{ background: token.colorBgContainer, color: token.colorText }}
        items={getItems(props.data)}
      />
      <Modal
        open={modalType !== null}
        title={title}
        cancelText="再想想"
        okText="确认"
        width={350}
        onCancel={() => setModalType(null)}
        onOk={handleConfirm}
      >
        <Form form={form}>
          <Form.Item
            name="name"
            rules={[
              { required: true, message: `${title}不能为空` },
              { max: 10, message: "不能超过十个字符" },
            ]}
          >
            <Input placeholder={placeholder} />
          </Form.Item>
        </Form>
      </Modal>
    </div>
  );
};
