import { useCallback, useEffect, useState } from "react";
import "./index.less";
import { Button, Form, Input, Modal, message } from "antd";
import { PlusOutlined } from "@ant-design/icons";
import { useDomain } from "@/hook/useDomain";
import { LabelTree } from "./LabelTree";
import {
  CategoryItemType,
  LabelCategoryType,
  LabelType,
} from "./LabelTree/interface";

interface LabelProps {
  children?: JSX.Element;
}

export const Label = (props: LabelProps) => {
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [createLabelCategoryForm] = Form.useForm<{ name: string }>();
  const { labelDomainService } = useDomain();
  const [labelList, setLabelList] = useState<CategoryItemType[]>([]);
  const [messageApi, contextHolder] = message.useMessage();

  const createLabelHandler = async () => {
    try {
      await createLabelCategoryForm.validateFields();
      const { name } = createLabelCategoryForm.getFieldsValue();
      labelDomainService.createNewLabelCategory({ name }).then((res) => {
        const { code } = res;
        if (code === 0) {
          renderLabelList();
          createLabelCategoryForm.resetFields();
          setIsModalOpen(false);
        }
      });
    } catch (error) {
      console.log(error);
    }
  };

  const renderLabelList = useCallback(async () => {
    const { code, data } = await labelDomainService.findLabelCategory();
    if (code === 0) setLabelList(data as CategoryItemType[]);
  }, [labelDomainService]);

  useEffect(() => {
    renderLabelList();
  }, [renderLabelList]);

  const onCreateLabelHandler = ({ name, categoryId }: LabelType) => {
    labelDomainService
      .createNewLabel({ name, categoryId: categoryId! })
      .then((res) => {
        const { code } = res;
        if (code === 0) {
          renderLabelList();
          messageApi.success(`标签 ${name} 添加成功`);
        }
      });
  };

  const onUpdateLabelHandler = ({ name, id }: LabelType) => {
    labelDomainService.updateLabel({ name, id: id! }).then((res) => {
      const { code } = res;
      if (code === 0) {
        renderLabelList();
        messageApi.success(`标签 ${name} 修改成功`);
      }
    });
  };

  const onDeleteLabelHandler = ({ id, name }: LabelType) => {
    labelDomainService.deleteLabel({ id: id! }).then((res) => {
      const { code } = res;
      if (code === 0) {
        renderLabelList();
        messageApi.success(`标签 ${name} 修改成功`);
      }
    });
  };

  const onUpdateCategoryHandler = ({ name, id }: LabelCategoryType) => {
    labelDomainService.updateCategory({ name, id }).then((res) => {
      const { code } = res;
      if (code === 0) {
        renderLabelList();
        messageApi.success(`${name} 分类修改成功`);
      }
    });
  };

  return (
    <div className="label-container">
      {contextHolder}
      <aside>
        <Button
          block
          type="primary"
          icon={<PlusOutlined />}
          style={{ marginBottom: 10 }}
          onClick={() => setIsModalOpen(true)}
        >
          标签分类
        </Button>
        <Modal
          title="创建标签分类"
          open={isModalOpen}
          onOk={createLabelHandler}
          width={350}
          okText="确认"
          cancelText="再想想"
          onCancel={() => setIsModalOpen(false)}
        >
          <Form form={createLabelCategoryForm} style={{ marginTop: 20 }}>
            <Form.Item
              name="name"
              rules={[
                { required: true, message: "标签分类名称不能为空" },
                { max: 10, message: "不能超过十个字符" },
              ]}
            >
              <Input placeholder="请输入标签分类名称" autoComplete="off" />
            </Form.Item>
          </Form>
        </Modal>
        <LabelTree
          data={labelList}
          onCreateLabel={onCreateLabelHandler}
          onUpdateLabel={onUpdateLabelHandler}
          onDeleteLabel={onDeleteLabelHandler}
          onUpdateCategory={onUpdateCategoryHandler}
        />
      </aside>
      <main>标签相关数据</main>
    </div>
  );
};
